	subroutine gibbs(iseed)
      implicit real*8 (a-h, o-z)
	integer, parameter :: ni = 590, nj = 4, nk = 3
	integer, parameter :: nr = 4+3, nc = nr+4
      integer, parameter :: nx = nk*nc, nu = nk*nr
      integer, parameter :: nnc = 5

      integer imiss(ni,nj)
      real*8 y(ni,nj,nk)
      real*8 xstar(ni,nj,nk,nx), ustar(ni,nj,nk,nu)
      real*8 betaa(nx), rstar(ni,nu)
      real*8 theta(nk), xi(ni,nj,nk), exi
      real*8 lambda(ni,nj), sigma(nj,nk,nk)
      real*8 vv, omega(nu,nu)
                             
      common /vimiss/imiss
      common /vy/y
      common /vxstar/xstar
      common /vustar/ustar

      common /vbetaa/betaa
      common /vrstar/rstar
      common /vtheta/theta
      common /vxi/xi
      common /vexi/exi
      common /vlambda/lambda
      common /vsigma/sigma
      common /vvv/vv          
      common /vomega/omega     
            
	call gibbs_rstar(iseed)

	call gibbs_xi(iseed)

	call gibbs_lambda(iseed)

	end subroutine

      
	subroutine gibbs_xi(iseed)
      implicit real*8 (a-h, o-z)
	integer, parameter :: ni = 590, nj = 4, nk = 3
	integer, parameter :: nr = 4+3, nc = nr+4
      integer, parameter :: nx = nk*nc, nu = nk*nr
      integer, parameter :: nnc = 5

      integer imiss(ni,nj)
      real*8 y(ni,nj,nk)
      real*8 xstar(ni,nj,nk,nx), ustar(ni,nj,nk,nu)
      real*8 betaa(nx), rstar(ni,nu)
      real*8 theta(nk), xi(ni,nj,nk), exi
      real*8 lambda(ni,nj), sigma(nj,nk,nk)
      
      real*8 sigmaj(nk,nk), sigmaj_inv(nk,nk)
      real*8 ystar(nk), xb, ub
                    
      real*8 tstar(nk,nk), ts(nk,nk), tst(nk,nk), tsy(nk)

      real*8 hmean(nk), hsigma(nk,nk)
      real*8 amean, asigma, xmean, xsigma, trim, rv
      
      logical la, lb
      
      common /vimiss/imiss
      common /vy/y
      common /vxstar/xstar
      common /vustar/ustar

      common /vbetaa/betaa
      common /vrstar/rstar
      common /vtheta/theta
      common /vxi/xi
      common /vexi/exi
      common /vlambda/lambda
      common /vsigma/sigma

      external ytuvn, dlinrg
     
      do k1 = 1, nk
        do k2 = 1, nk
            tstar(k1,k2) = 0.d0
        enddo
        tstar(k1,k1) = theta(k1)
      enddo
      
      do i = 1, ni

        do j = 1, nj
                   
            if (imiss(i,j) .eq. 1) then
                
	          do k1 = 1, nk
	              do k2 = 1, nk
	                  sigmaj(k1,k2) = sigma(j,k1,k2)
	              enddo	      	      
	          enddo       
	          
	          call dlinrg(nk, sigmaj, nk, sigmaj_inv, nk)

                do k = 1, nk
                
                    xb = 0.d0                        
                    do jj = 1, nx
                        xb = xb + xstar(i,j,k,jj)*betaa(jj)
                    enddo

                    ub = 0.d0                        
                    do jj = 1, nu
                        ub = ub + ustar(i,j,k,jj)*rstar(i,jj)
                    enddo
                
                    ystar(k) = y(i,j,k) - xb - ub + theta(k)*exi
     
                enddo

                do k1 = 1, nk
                    do k2 = 1, nk
                        ts(k1,k2) = 0.d0
                        do kk = 1, nk
                            ts(k1,k2) = ts(k1,k2) 
     +                                + tstar(kk,k1)
     +                                  *sigmaj_inv(kk,k2)
                        enddo
                    enddo
                enddo

                do k1 = 1, nk

                    tsy(k1) = 0.d0                
                    do k2 = 1, nk
                        tsy(k1) = tsy(k1) 
     +                          + ts(k1,k2)*ystar(k2)
                    enddo
                
                    do k2 = 1, nk
                        tst(k1,k2) = 0.d0
                        do kk = 1, nk
                            tst(k1,k2) = tst(k1,k2) 
     +                                 + ts(k1,kk)*tstar(kk,k2)
                        enddo
                    enddo
                    
                enddo

                do k1 = 1, nk
        
                    hmean(k1) = lambda(i,j)*tsy(k1) - 1.d0
                   
                    do k2 = 1, nk
                
                        hsigma(k1,k2) = lambda(i,j)*tst(k1,k2)
                    
                    enddo

                    hsigma(k1,k1) = hsigma(k1,k1) + 1.d0
                            
                enddo      

                do ir = 1, 10
                do k1 = 1, nk
        
                    asigma = hsigma(k1,k1)
            
                    amean = hmean(k1)
                    do k2 = 1, nk
                        if (k2 .ne. k1) then
                            amean = amean - hsigma(k1,k2)*xi(i,j,k2)
                        endif
                    enddo
        
                    xmean = amean/asigma
                    xsigma = 1.d0/asigma
                        
                    trim = -xmean/dsqrt(xsigma)
            
                    la = .false. ; lb = .true.

		            call rnset(iseed)
		            rv = ytuvn(trim, trim, la, lb, iseed)
		            call rnget(iseed)
		    
		            xi(i,j,k1) = xmean + rv*dsqrt(xsigma) 

                enddo
                enddo

            else
        
                do k = 1, nk
                    xi(i,j,k) = -999.d0
                enddo
                    
            endif
            
        enddo
                                
      enddo
      
      end subroutine      
     
      
	subroutine gibbs_lambda(iseed)
      implicit real*8 (a-h, o-z)
	integer, parameter :: ni = 590, nj = 4, nk = 3
	integer, parameter :: nr = 4+3, nc = nr+4
      integer, parameter :: nx = nk*nc, nu = nk*nr
      integer, parameter :: nnc = 5

      integer imiss(ni,nj)
      real*8 y(ni,nj,nk)
      real*8 xstar(ni,nj,nk,nx), ustar(ni,nj,nk,nu)
      real*8 betaa(nx), rstar(ni,nu)
      real*8 theta(nk), xi(ni,nj,nk), exi
      real*8 lambda(ni,nj), sigma(nj,nk,nk), vv
      
      real*8 sigmaj(nk,nk), sigmaj_inv(nk,nk)
      real*8 ystar(nk), xb, ub, temp
      real*8 shape, scale, rv
          
      common /vimiss/imiss
      common /vy/y
      common /vxstar/xstar
      common /vustar/ustar

      common /vbetaa/betaa
      common /vrstar/rstar
      common /vtheta/theta
      common /vxi/xi
      common /vexi/exi
      common /vlambda/lambda
      common /vsigma/sigma

      common /vvv/vv          

      external dlinrg, dblinf, drngam
                
      do j = 1, nj   

	  do k1 = 1, nk
	      do k2 = 1, nk
	          sigmaj(k1,k2) = sigma(j,k1,k2)
	      enddo	      	      
	  enddo        

	  call dlinrg(nk, sigmaj, nk, sigmaj_inv, nk)

        do i = 1, ni

            if (imiss(i,j) .eq. 1) then
                          
                do k = 1, nk

                    xb = 0.d0                        
                    do jj = 1, nx
                        xb = xb + xstar(i,j,k,jj)*betaa(jj)
                    enddo

                    ub = 0.d0                        
                    do jj = 1, nu
                        ub = ub + ustar(i,j,k,jj)*rstar(i,jj)
                    enddo
                
                    ystar(k) = y(i,j,k) - xb - ub
     +                       - theta(k)*(xi(i,j,k) - exi)
                enddo        

                temp = dblinf(nk,nk,sigmaj_inv,nk,ystar,ystar)
            
                shape = (vv + dfloat(nk))/2.d0
                scale = (1.d0 + temp)/2.d0

	          call rnset(iseed)
	          call drngam(1, shape, rv)
	          call rnget(iseed)

                lambda(i,j) = rv/scale      

            else

                lambda(i,j) = -999.d0 
                        
            endif
            
        enddo
        
      enddo
                                    
      end subroutine
      

	subroutine gibbs_rstar(iseed)
      implicit real*8 (a-h, o-z)
	integer, parameter :: ni = 590, nj = 4, nk = 3
	integer, parameter :: nr = 4+3, nc = nr+4
      integer, parameter :: nx = nk*nc, nu = nk*nr
      integer, parameter :: nnc = 5

      integer imiss(ni,nj)
      real*8 y(ni,nj,nk)
      real*8 xstar(ni,nj,nk,nx), ustar(ni,nj,nk,nu)
      real*8 betaa(nx), rstar(ni,nu)
      real*8 theta(nk), xi(ni,nj,nk), exi
      real*8 lambda(ni,nj), sigma(nj,nk,nk), omega(nu,nu)
      
      real*8 sigmaj(nk,nk), sigmaj_inv(nk,nk)
      real*8 omega_inv(nu,nu)
      real*8 us(nu,nk), usu(nu,nu), usy(nu)
      real*8 ystar(nk), xb
      
      real*8 hmean(nu), hsigma(nu,nu)
      real*8 amean(nu), asigma(nu,nu)
	real*8 tol, rsig(nu,nu), rv(nu)
      
      common /vimiss/imiss
      common /vy/y
      common /vxstar/xstar
      common /vustar/ustar

      common /vbetaa/betaa
      common /vrstar/rstar
      common /vtheta/theta
      common /vxi/xi
      common /vexi/exi
      common /vlambda/lambda
      common /vsigma/sigma
      common /vomega/omega     
      
      external dlinrg, dmach, dchfac, drnmvn

	call dlinrg(nu, omega, nu, omega_inv, nu)

      do i = 1, ni

        do j1 = 1, nu
            hmean(j1) = 0.d0
            do j2 = 1, nu
                hsigma(j1,j2) = 0.d0
            enddo
        enddo

        do j = 1, nj
        
            if (imiss(i,j) .eq. 1) then

	          do k1 = 1, nk
	              do k2 = 1, nk
	                  sigmaj(k1,k2) = sigma(j,k1,k2)
	              enddo	      	      
	          enddo        

	          call dlinrg(nk, sigmaj, nk, sigmaj_inv, nk)

                do j1 = 1, nu
                    do j2 = 1, nk
                        us(j1,j2) = 0.d0
                        do jj = 1, nk
                            us(j1,j2) = us(j1,j2) 
     +                                + ustar(i,j,jj,j1)
     +                                  *sigmaj_inv(jj,j2)
                        enddo
                    enddo
                enddo

                do k = 1, nk

                    xb = 0.d0                        
                    do jj = 1, nx
                        xb = xb + xstar(i,j,k,jj)*betaa(jj)
                    enddo
                
                    ystar(k) = y(i,j,k) - xb 
     +                       - theta(k)*(xi(i,j,k) - exi)
     
                enddo

                do j1 = 1, nu

                    usy(j1) = 0.d0
                    do j2 = 1, nk
                        usy(j1) = usy(j1) 
     +                          + us(j1,j2)*ystar(j2)
                    enddo
                
                    do j2 = 1, nu
                        usu(j1,j2) = 0.d0
                        do jj = 1, nk
                            usu(j1,j2) = usu(j1,j2) 
     +                                 + us(j1,jj)
     +                                   *ustar(i,j,jj,j2)
                        enddo
                    enddo
                    
                enddo

                do j1 = 1, nu
                    hmean(j1) = hmean(j1) 
     +                        + lambda(i,j)*usy(j1)
                    do j2 = 1, nu
                        hsigma(j1,j2) = hsigma(j1,j2) 
     +                                + lambda(i,j)*usu(j1,j2)
                    enddo
                enddo

            endif
            
        enddo

        do j1 = 1, nu
            do j2 = 1, nu
                hsigma(j1,j2) = hsigma(j1,j2) + omega_inv(j1,j2) 
            enddo
        enddo
       
	  call dlinrg(nu, hsigma, nu, asigma, nu)
	
	  do j1 = 1, nu
	      amean(j1) = 0.d0
		    do j2 = 1, nu
		        amean(j1) = amean(j1) 
     +                    + asigma(j1,j2)*hmean(j2)
		    enddo
	  enddo

	  tol = 100.d0*dmach(4)
	  call dchfac(nu, asigma, nu, tol, irank, rsig, nu)
	  call rnset(iseed)
	  call drnmvn(1, nu, rsig, nu, rv, 1)
	  call rnget(iseed)      
	
	  do j = 1, nu
	      rstar(i,j) = amean(j) + rv(j)
	  enddo      
      
      enddo

      end subroutine      
      